#!/bin/bash
set -o pipefail

APT_SRC_BASE=/etc/apt/sources.list.d
FORCE_SSL=${FORCE_SSL-0}
PROTO=http

if [ "$FORCE_SSL" = "1" ] ; then
    PROTO=https
fi

BASE_URL=${PROTO}://apt.cvedia.com

# first pass
ARCH=${ARCH-$(uname -p)}

# second pass
if [ -z "$ARCH" ] || [ "$ARCH" == "unknown" ] ; then
    if [ -n "$(command -v lscpu)" ] ; then
        x=$(lscpu | head -n1 | rev | cut -d' ' -f1 | rev)
    fi

    if [ -z "$x" ] && [ -n "$(command -v uname)" ] ; then
        x=$(uname -m)
    fi

    if [ -z "$x" ] || [ "$x" == "unknown" ]; then
        bits=0
        is_arm=0
        
        if [ -n "$(command -v getconf)" ] ; then
            bits=$(getconf LONG_BIT)
        fi
        
        if [ -f "/proc/cpuinfo" ] ; then
            is_arm=$(grep -ci "arm" /proc/cpuinfo)
        fi

        if [ "$is_arm" -gt "0" ] && [ "$bits" == "64" ] ; then
            ARCH=aarch64
        elif [ "$is_arm" -gt "0" ] ; then # 32 bit arm
            ARCH=armv7l
        else # not arm
            ARCH=x86_64
        fi
    fi
    
    ARCH=$x
fi

if [ ! -d "$APT_SRC_BASE" ] ; then
    echo "Error: $APT_SRC_BASE does not exist, aborting!"
    exit 1
fi

if [ -n "$(command -v apt)" ] ; then
    if [ -z "$(command -v curl)" ] ; then
        apt update && apt install curl -y
        RT=$?
        if [ "$RT" -ne "0" ] ; then
            echo "Failed to install curl, aborting!"
            exit 1
        fi
    fi

    if [[ $EUID -ne 0 ]]; then
        echo "Please run this script as root or with sudo"
        exit 1
    fi

    echo "Adding cvedia repository key..."
    
    # ubuntu 22+
    if [ -n "$(command -v gpg)" ] ; then
        KEY_PATH=/etc/apt/trusted.gpg.d/cvedia.gpg
        if [ -e "$KEY_PATH" ] ; then
            rm -rf "$KEY_PATH"
        fi

        mkdir -p /etc/apt/trusted.gpg.d &> /dev/null
        curl --fail -sqko - ${BASE_URL}/key.gpg | gpg --dearmour -o $KEY_PATH
        RT=$?
    # old ubuntu / deb
    elif [ -n "$(command -v apt-key)" ] ; then
        curl --fail -sqko - ${BASE_URL}/key.gpg | apt-key add -
        RT=$?
    else
        echo "-- No tool available (gpg or apt-key) to add cvedia repository key, aborting!"
        exit 1
    fi

    if [ "$RT" -ne "0" ] ; then
        echo "Failed to add cvedia repository key, aborting!"
        exit 1
    fi

    APT_LST_FN=$APT_SRC_BASE/cvedia.list
    
    if [ -e "$APT_LST_FN" ] ; then
        echo "WARN: CVEDIA apt list file already exist, overwriting..."
    fi
    
    if [ -n "$(command -v dpkg-query)" ] ; then
        LIBC6=$(dpkg-query -W -f='${Version}' libc6 | cut -d'-' -f1 | tr -d '.')
    elif [ -n "$(command -v ldd)" ] ; then
        LIBC6=$(ldd --version | head -n1 | rev | cut -d' ' -f1 | rev | tr -d '.')
    fi

    if [ -z "$LIBC6" ] ; then
        echo "WARN: Failed to determine libc6 version, assuming legacy"
        LIBC6=000
    fi

    if [[ "${#LIBC6}" -gt 3 ]] ; then
        LIBC6=${LIBC6:0:3}
    elif [[ "${#LIBC6}" -lt 3 ]] ; then
        LIBC6=$(printf "%-3s" $LIBC6 | tr ' ' '0')
    fi

    echo "Adding cvedia repository..."

    {
        echo "# added by get.cvedia.com @ $(date)"
        echo ""
        
        #shellcheck disable=SC2235
        if ( [ "$ARCH" = "x86_64" ] && [ "$LIBC6" -ge 234 ] ) || ( [ "$ARCH" != "x86_64" ] && [ "$LIBC6" -ge 231 ] ) ; then
            echo "deb ${BASE_URL} cvedia main"
            echo "#deb ${BASE_URL} cvedia legacy"
        else
            echo "# old version of libc6 detected, using legacy repository"
            echo "#deb ${BASE_URL} cvedia main"
            echo "deb ${BASE_URL} cvedia legacy"
        fi
        
        if [ -n "$POOLS" ] ; then
            for p in $POOLS; do
                echo "deb ${BASE_URL} cvedia $p"
            done
        fi
    } > $APT_LST_FN

    echo "Updating local package list..."
    
    apt update
    
    echo "CVEDIA repository added, you can now install CVEDIA-RT with: apt install cvedia-rt"
    echo "To revert this change delete $APT_LST_FN"
else
    echo "APT not found, aborting!"
    echo "CVEDIA repositories are compatible with APT-based systems only."
    exit 1
fi
